﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace ConsoleApplication1
{
    class Program
    {
        static void Main(string[] args)
        {
            int[] list = new int[] { 200, 300, 500, 600 };

            Combination c = new Combination(list);

            foreach (var item in c.Combinations())
            {
                PrintElems(item);
            }
        }

        private static void PrintElems(int[] elems)
        {
            Console.Write("{ ");

            foreach (var elem in elems)
            {
                Console.Write(elem + ", ");
            }

            Console.WriteLine(" }");
        }
    }

    public class Node
    {
        public readonly Node Parent;
        public readonly int Index;

        [DebuggerHidden]
        public Node(Node parent, int index)
        {
            this.Parent = parent;
            this.Index = index;
        }

        public override string ToString()
        {
            Node current = this;

            StringBuilder sb = new StringBuilder();
            sb.Append("{ ");
            while (current != null)
            {
                sb.Append(current.Index + ", ");
                current = current.Parent;
            }
            sb.Append(" }");

            return sb.ToString();
        }
    }

    public class Combination
    {
        readonly int _depth;
        readonly int[] _sourceList;

        List<Node> _leafNodes = new List<Node>();

        public Combination(int [] elems)
        {
            _sourceList = elems;
            _depth = _sourceList.Length;

            Prepare();
        }

        void Prepare()
        {
            VisitElement(1, new Node(null, 1));
            VisitElement(1, new Node(null, 0));
        }

        private void VisitElement(int depth, Node node)
        {
            if (depth == _depth)
            {
                _leafNodes.Add(node);
                return;
            }

            VisitElement(depth + 1, new Node(node, 1));
            VisitElement(depth + 1, new Node(node, 0));
        }

        internal IEnumerable<int []> Combinations()
        {
            foreach (Node leaf in _leafNodes)
            {
                Node node = leaf;
                List<int> elems = new List<int>();
                
                int index = 0;
                while (node != null)
                {
                    if (node.Index == 1)
                    {
                        elems.Add(_sourceList[index]);
                    }

                    index++;
                    node = node.Parent;
                }

                yield return elems.ToArray();
            }
        }
    }
}
